import 'dart:io';

import 'package:jt_flutter_cli/command_executor.dart';
import 'package:jt_flutter_cli/core/const/command_const.dart';
import 'package:jt_flutter_cli/core/util/util.dart';
import 'package:jt_flutter_cli/core/util/validator_util.dart';

void main(List<String> arguments) async {
  if (!ValidatorUtil.isCommandValid(arguments: arguments)) {
    Utility.commandError();
    exit(1);
  }

  if (arguments[0].toLowerCase() == kCreate) {
    if (arguments[1].toLowerCase() == kFeatureCommand) {
      /// create new feature (create feature name_of_feature)
      await CommandExecutor.createFeature(featureName: arguments[2]);
    } else if (arguments[1].toLowerCase() == kWidget) {
      /// create widget (create widget name_of_widget)
      await CommandExecutor.createWidget(widgetName: arguments[2]);
    } else if (arguments[1].toLowerCase() == kModel) {
      /// create model (create model {model_name} {feature_name} {json_file_path})
      CommandExecutor.createModel(
        modelName: arguments[2],
        featureName: arguments[3],
        jsonFilePath: arguments[4],
      );
    } else if (arguments[1].toLowerCase() == kAssets) {
      /// create assets (create assets {3x image path} {directory name in assets/images/ (This is where image will be saved)})
      CommandExecutor.generateAssets(
        inputImagePath: arguments[2],
        imageToSaveDirectoryName: arguments[3],
      );
    }
  } else if (arguments[0].toLowerCase() == kAdd) {
    if ((arguments[1] == kGetApiCall ||
        arguments[1] == kPostApiCall ||
        arguments[1] == kMultipartApiCall)) {
      /// Add API call
      await CommandExecutor.generateApiCallCode(
        apiMethod: arguments[1],
        apiEndPoint: arguments[2],
        apiName: arguments[3],
      );
    }
  }

  ///Code format
  await Utility.formatCode();
}
