import 'package:jt_flutter_cli/core/util/model_util/helpers.dart';
import 'package:jt_flutter_cli/core/util/util.dart';

const String kStartComment = '/*';
const String kEndComment = '*/';
const String kWhitespaces = '';
const String kBasicStructureLowerCase = 'basic_structure';
const String kBasicStructurePascalCase = 'BasicStructure';
const String kBasicStructureCamelCase = 'basicStructure';
const String kBasicWidget = 'BasicWidget';
const String kBasicWidgetLowerCase = 'basic_widget';
const String kName = 'name';
const String kFlutterCleanArchitecture = 'flutter_clean_architecture';
const String kRequestModel = '_request_model';
const String kResponseModel = '_response_model';
const String kOverride = '@override';
const String kFutureApiResponse = 'Future<ApiResponse>';
const String kCall = 'call';
const String kRemoteDataSource = 'remoteDataSource.';
const String kExecuteGet = 'executeGet';
const String kQueryParameter = 'queryParameters:';
const String kToJson = '.toJson()';
const String kPath = 'path:';
const String kExecutePost = 'executePost';
const String kRequestBody = 'requestBody:';
const String kFutureFailureSuccess = 'Future<Either<Failure, Success>>';
const String kCommonCap = 'Common';
const String kUseCasesPrefixUnderscore = '_usecase';
const String kParams = 'params';
const String kNoRequest = 'NoRequest';
const String kFinal = 'final';
const String kUseCasePascal = 'UseCase';
const String kReturn = 'return';
const String kExecuteMultipart = 'executeMultipart';
const String kDartIoImport = "import 'dart:io';";

class CodeHandler {
  String getDiGetterStatement({
    required String pascalCaseFeatureName,
  }) =>
      "\n${pascalCaseFeatureName}Repo get${pascalCaseFeatureName}Repo();";

  String getMockDiGetterStatement({
    required String pascalCaseFeatureName,
  }) =>
      "\n@override\n${pascalCaseFeatureName}Repo get${pascalCaseFeatureName}Repo() => Mock${pascalCaseFeatureName}RepoImpl(sl(), sl());";

  String getProdDiGetterStatement({
    required String pascalCaseFeatureName,
  }) =>
      "\n@override\n${pascalCaseFeatureName}Repo get${pascalCaseFeatureName}Repo() => ${pascalCaseFeatureName}RepoImpl(sl(), sl());";

  String getInjectionContainerStatement({
    required String pascalCaseFeatureName,
  }) =>
      "\nsl.registerLazySingleton<${pascalCaseFeatureName}DataSource>("
      "() => ${pascalCaseFeatureName}DataSourceImpl(remoteDataSource: sl()),"
      ");";

  String getDataSourceStatementWithRequestModel({
    required String pascalCaseApiName,
    required String requestModelName,
  }) =>
      "  Future<ApiResponse> call$pascalCaseApiName (${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)});";

  String getDataSourceStatementWithoutRequestModel({
    required String pascalCaseApiName,
  }) =>
      "  Future<ApiResponse> call$pascalCaseApiName();";

  List<String> getDataSourceImplStatementForGetMethodWithRequestModel({
    required String pascalCaseApiName,
    required String requestModelName,
    required String apiMethodName,
    required String apiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureApiResponse $kCall$pascalCaseApiName(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)}) =>",
        "      $kRemoteDataSource$kExecuteGet($kPath ${apiName.toUpperCase()}_API, \n      $kQueryParameter ${Utility.camelCaseFirstLower(input: requestModelName)}$kToJson,);"
      ];

  List<String> getDataSourceImplStatementForPostMethodWithRequestModel({
    required String pascalCaseApiName,
    required String requestModelName,
    required String apiMethodName,
    required String apiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureApiResponse $kCall$pascalCaseApiName(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)}) =>",
        "      $kRemoteDataSource$kExecutePost($kPath ${apiName.toUpperCase()}_API, \n      $kRequestBody ${Utility.camelCaseFirstLower(input: requestModelName)}$kToJson,);"
      ];

  List<String> getDataSourceImplStatementForMultipartMethod({
    required String pascalCaseApiName,
    required String requestModelName,
    required String apiMethodName,
    required String apiName,
    required List<String> multipartFileKeys,
  }) {
    List<String> multipartFileKeyList = [];
    List<String> multipartFilePathList = [];
    for (int index = 0; index < multipartFileKeys.length; index++) {
      String fieldName =
          "${Utility.camelCaseFirstLower(input: multipartFileKeys[index])}File";
      multipartFileKeyList.add(
        multipartFileKeys[index],
      );
      multipartFilePathList.add(
        "\${${Utility.camelCaseFirstLower(input: requestModelName)}.$fieldName!.path}",
      );
    }

    return [
      "  $kOverride",
      "  $kFutureApiResponse $kCall$pascalCaseApiName(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)}){ \n "
          "   \nList<FileUploadRequestModel> fileUploadRequestModelList =[];\n"
          '   \nString multipartFileKey = "${multipartFileKeyList.join(",")}";\n'
          '   \nString multipartFilePath = "${multipartFilePathList.join(",")}";\n'
          '   \nList<String> multipartFileKeyList = multipartFileKey.split(",");\n'
          '   \nList<String> multipartFilePathList = multipartFilePath.split(",");\n'
          'for(int index = 0; index < multipartFileKeyList.length;index++){\n'
          "   String fileParameter = multipartFileKeyList[index];"
          "   \nString filePath = multipartFilePathList[index];\n"
          '   String fileName = filePath.split("/").last;\n'
          "fileUploadRequestModelList.add(      FileUploadRequestModel(fileName: fileName, filePath: filePath, contentType: '\${lookupMimeType(fileName)}', imageFieldKey: fileParameter.toString(),),);\n",
      "   \nif (${Utility.camelCaseFirstLower(input: requestModelName)}$kToJson.containsKey(multipartFileKeyList[index])) { \n "
          "    ${Utility.camelCaseFirstLower(input: requestModelName)}$kToJson.remove(multipartFileKeyList[index]);"
          "   } \n"
          "}"
          "   \nMap<String,String>  convertedMap = {};  \n"
          "   \n${Utility.camelCaseFirstLower(input: requestModelName)}$kToJson.forEach((key, value) { \n"
          "   convertedMap[key] = value.toString();"
          "   },); \n"
          "   \n$kReturn  $kRemoteDataSource$kExecuteMultipart($kPath ${apiName.toUpperCase()}_API,\n      fileUploadRequestModelList:fileUploadRequestModelList, \n      $kRequestBody convertedMap,);\n}"
    ];
  }

  List<String> getDataSourceImplStatementForGetMethodWithoutRequestModel({
    required String pascalCaseApiName,
    required String apiMethodName,
    required String apiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureApiResponse $kCall$pascalCaseApiName() =>",
        "      $kRemoteDataSource$kExecuteGet($kPath ${apiName.toUpperCase()}_API);"
      ];

  List<String> getDataSourceImplStatementForPostMethodWithoutRequestModel({
    required String pascalCaseApiName,
    required String apiMethodName,
    required String apiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureApiResponse $kCall$pascalCaseApiName() =>",
        "      $kRemoteDataSource$kExecutePost($kPath ${apiName.toUpperCase()}_API);"
      ];

  String getRepoStatementWithRequestModel({
    required String requestModelName,
    required String pascalCaseApiName,
  }) =>
      "  $kFutureFailureSuccess $kCall$pascalCaseApiName(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)});";

  String getRepoStatementWithoutRequestModel({
    required String pascalCaseApiName,
  }) =>
      "  $kFutureFailureSuccess $kCall$pascalCaseApiName();";

  List<String> getRepoImplStatementWithRequestModel({
    required String pascalCaseApiName,
    required String requestModelName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureFailureSuccess $kCall$pascalCaseApiName(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)}) =>",
        "      baseApiMethod(() => $kCall${pascalCaseApiName}ApiCall(${Utility.camelCaseFirstLower(input: requestModelName)}));",
        "",
        "  $kFutureFailureSuccess $kCall${pascalCaseApiName}ApiCall(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)}) async {",
        "    ApiResponse response = await dataSource.call$pascalCaseApiName(${Utility.camelCaseFirstLower(input: requestModelName)});",
        "",
        "    if (!response.status!) {",
        "      return Left(",
        "        ServerFailure(",
        "          errorMessage: response.message!,",
        "          statusCode: response.statusCode ?? 0,",
        "        ),",
        "      );",
        "    } else {"
      ];

  List<String> getRepoImplStatementWithoutRequestModel({
    required String pascalCaseApiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureFailureSuccess $kCall$pascalCaseApiName() =>",
        "      baseApiMethod(() => $kCall${pascalCaseApiName}ApiCall());",
        "",
        "  $kFutureFailureSuccess $kCall${pascalCaseApiName}ApiCall() async {",
        "    ApiResponse response = await dataSource.call$pascalCaseApiName();",
        "",
        "    if (!response.status!) {",
        "      return Left(",
        "        ServerFailure(",
        "          errorMessage: response.message!,",
        "          statusCode: response.statusCode ?? 0,",
        "        ),",
        "      );",
        "    } else {"
      ];

  List<String> getMockRepoImplStatementWithRequestModel({
    required String pascalCaseApiName,
    required String requestModelName,
    required String apiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureFailureSuccess $kCall$pascalCaseApiName(${camelCase(requestModelName)} ${Utility.camelCaseFirstLower(input: requestModelName)}) async {",
        "    ApiResponse apiResponse = await loadFromAssetData(${apiName.toUpperCase()}_API);",
      ];

  List<String> getMockRepoImplStatementWithoutRequestModel({
    required String pascalCaseApiName,
    required String apiName,
  }) =>
      [
        "  $kOverride",
        "  $kFutureFailureSuccess $kCall$pascalCaseApiName() async {",
        "    ApiResponse apiResponse = await loadFromAssetData(${apiName.toUpperCase()}_API);",
      ];

  List<String> getInjectionContainerStatements({
    required String pascalCase,
    required String featureName,
  }) =>
      [
        "  $kWhitespaces",
        "  sl.registerLazySingleton("
            "() => ${pascalCase}UseCase(getter.get${camelCase(featureName)}Repo()),"
            ");"
      ];

  List<String> getStateStatementsWithRequestModel({
    required String pascalCaseApiName,
    required String responseModelName,
    required String featureNamePascalCase,
  }) =>
      [
        "",
        "class ${pascalCaseApiName}State extends ${featureNamePascalCase}State {",
        "  final ${camelCase(responseModelName)} response;",
        "  final String message;",
        "",
        "  ${pascalCaseApiName}State({",
        "    required this.response,",
        "    required this.message",
        "  });",
        "}",
        "",
        "class ${pascalCaseApiName}ErrorState extends ${featureNamePascalCase}State {",
        "  final String message;",
        "  final int statusCode;",
        "",
        "  ${pascalCaseApiName}ErrorState({",
        "    required this.message,",
        "    required this.statusCode,",
        "  });",
        "}",
      ];

  List<String> getStateStatementsWithoutRequestModel({
    required String pascalCaseApiName,
    required String featureNamePascalCase,
  }) =>
      [
        "",
        "class ${pascalCaseApiName}State extends ${featureNamePascalCase}State {",
        "  final String message;",
        "",
        "  ${pascalCaseApiName}State({",
        "    required this.message",
        "  });",
        "}",
        "",
        "class ${pascalCaseApiName}ErrorState extends ${featureNamePascalCase}State {",
        "  final String message;",
        "  final int statusCode;",
        "",
        "  ${pascalCaseApiName}ErrorState({",
        "    required this.message,",
        "    required this.statusCode,",
        "  });",
        "}",
      ];

  List<String> getBlocStatementsShimmerChoiceY({
    required String pascalCaseApiName,
    required String apiName,
    required String usCaseEvent,
  }) =>
      [
        "",
        "Future<void> ${Utility.camelCaseFirstLower(input: apiName)}Event(${pascalCaseApiName}Event event, Emitter emit) async {",
        "  emit(${pascalCaseApiName}DataLoadingState());",
        "  Either<Failure, Success> result = await $usCaseEvent;",
        "  await result.fold(",
        "    (failure) async {",
        "      if (!isUnAuthorised(failure)) {",
        "          emit(${pascalCaseApiName}ErrorState(message: failure.errorMessage, statusCode: failure.statusCode));",
        "      }",
        "    },",
        "    (data) async {",
      ];

  List<String> getBlocStatementsShimmerChoiceN({
    required String pascalCaseApiName,
    required String apiName,
    required String usCaseEvent,
  }) =>
      [
        "",
        "Future<void> ${Utility.camelCaseFirstLower(input: apiName)}Event(${pascalCaseApiName}Event event, Emitter emit) async {",
        "  apiBaseBlocObject.add(LoadApiEvent());",
        "  Either<Failure, Success> result = await $usCaseEvent;",
        "  await result.fold(",
        "    (failure) async {",
        "      apiBaseBlocObject.add(ErrorApiEvent(failure: failure));",
        "      emit(${pascalCaseApiName}ErrorState(message: failure.errorMessage, statusCode: failure.statusCode));",
        "    },",
        "    (data) async {",
        "      apiBaseBlocObject.add(LoadedApiEvent());"
      ];

  String getRepoStatementWithResponseModel({
    required String responseModelName,
  }) =>
      ""
      " if (response.data == null) {"
      "           return Right(Success(data: null, message: response.message));"
      "       } else {"
      "           ${camelCase(responseModelName)} responseData ="
      "                  ${camelCase(responseModelName)}.fromJson(response.data);"
      "           return Right(Success(data: responseData, message: response.message));"
      "       }"
      "    }"
      "  }";
  String getRepoStatementWithoutResponseModel() =>
      "       return Right(Success(data: null, message: response.message));"
      "    }"
      "  }";

  String getMockRepoStatementWithResponseModel({
    required String responseModelName,
  }) =>
      "     ${camelCase(responseModelName)} response ="
      "         ${camelCase(responseModelName)}.fromJson(apiResponse.data);"
      "    return Right(Success(data: response));"
      "  }";

  String getMockRepoStatementWithoutResponseModel() =>
      "       return Right(Success(data: null, message: apiResponse.message));"
      "  }";

  String getPageSearchString() =>
      "apiBaseBloc: BlocProvider.of<ApiBaseBloc>(context),";

  String getPageReplacementString({
    required String useCaseName,
  }) =>
      "apiBaseBloc: BlocProvider.of<ApiBaseBloc>(context),\n${useCaseName}UseCase: sl(),";

  String getEventStatementWithResponseModel({
    required String pascalCaseApiName,
    required String featureNamePascalCase,
    required String requestModelName,
  }) =>
      ""
      "class  ${pascalCaseApiName}Event extends  ${featureNamePascalCase}Event {"
      "  final ${camelCase(requestModelName)} request;"
      "  ${pascalCaseApiName}Event({required this.request});"
      "}";

  String getEventStatementWithoutResponseModel({
    required String pascalCaseApiName,
    required String featureNamePascalCase,
  }) =>
      ""
      "class ${pascalCaseApiName}Event extends ${featureNamePascalCase}Event {}";

  String getStateStatementShimmerChoiceY({
    required String pascalCaseApiName,
    required String featureNamePascalCase,
  }) =>
      ""
      "class ${pascalCaseApiName}DataLoadingState extends ${featureNamePascalCase}State {}"
      "";

  String getUseCaseEventNoRequest({
    required String useCaseName,
  }) =>
      "${Utility.camelCaseFirstLower(input: useCaseName)}(NoRequest())";

  String getUseCaseEventWithRequestModel({
    required String useCaseName,
  }) =>
      "${Utility.camelCaseFirstLower(input: useCaseName)}(event.request)";

  String getBlocStatementWithResponseModel({
    required String responseModelName,
    required String responseName,
    required String pascalCaseApiName,
  }) =>
      "      ${camelCase(responseModelName)} $responseName = data.data;"
      "      emit(${pascalCaseApiName}State(response: $responseName, message: data.message ?? ''));"
      "    },"
      "  );"
      "}";

  String getBlocStatementWithoutResponseModel({
    required String pascalCaseApiName,
  }) =>
      "      emit(${pascalCaseApiName}State(message: data.message ?? ''));"
      "    },"
      "  );"
      "}";

  List<String> getAddBlocAfterSuperApiBaseBloc({
    required String pascalCaseApiName,
    required String apiName,
  }) =>
      [
        "    on<${pascalCaseApiName}Event>(${Utility.camelCaseFirstLower(input: apiName)}Event);"
      ];

  String getBlocAppendAfterContainString({
    required String pascalCaseApiName,
  }) =>
      "super(apiBaseBloc";

  List<String> getAddBlocBeforeSuperApiBaseBloc({
    required String useCaseName,
  }) =>
      [
        "    required this.${Utility.camelCaseFirstLower(input: useCaseName)},",
      ];

  String getBlocSuperApiBaseBloc({
    required String pascalCaseApiName,
  }) =>
      "super(apiBaseBloc";

  String getExtendsBaseBloc({
    required String pascalCaseApiName,
  }) =>
      "extends BaseBloc";

  String getConstantStringAppend({
    required String apiType,
    required String apiName,
    required String endPoint,
  }) =>
      'const String ${apiName.toUpperCase()}_API = "$endPoint";';

  String getMockConstantStringAppend({
    required String apiName,
    required String endPoint,
  }) =>
      'const String ${apiName.toUpperCase()}_API = "\$basePath/$endPoint";';
}
