import 'dart:io';

import 'package:jt_flutter_cli/core/code/code_handler.dart';
import 'package:jt_flutter_cli/core/const/string_const.dart';
import 'package:jt_flutter_cli/core/util/util.dart';
import 'package:path/path.dart' as path;

const String kDart = ".dart";
const String kPubspecPath = "/pubspec.yaml";
const String kLib = 'lib';
const String kCore = '/core/';
const String kLibCore = '$kLib$kCore';
const String kFeatures = '/features/';
const String kFeature = '/feature/';
const String kLibFeatures = '$kLib$kFeatures';
const String kCreateFeatureBasicStructure = 'create_feature/basic_structure';
const String kUseCaseBasicStructure = 'add_api_call/usecases';
const String kLibCoreDi = '${kLibCore}di/';
const String kDomainRepositories = '/domain/repositories/';
const String kModelPath = '/data/model';
const String kWidgetPath = '/widgets/';
const String kWidgetWithoutEndSlashPath = '/widgets';
const String kCommonWidgetPath = '${kLibCore}ui$kWidgetPath';
const String kScreenPath = '/presentation/screen';
const String kModel = '/data/model';
const String kDiGetter = '/di_getter.dart';
const String kMockDiGetter = '/mock_di_getter.dart';
const String kProdDiGetter = '/prod_di_getter.dart';
const String kInjectionContainer = '/injection_container.dart';
String kImportPackageWithName =
    "import 'package:${Utility.getPubspecInfo() ?? ''}";
const String kDataRepository = '/data/repository/';
const String kDataDataSource = '/data/data_source/';
const String kWidgetSnippetPath = 'create_widget/basic_widget';
const String kAssetsMockJson = 'assets/mock_json/';
const String kJson = ".json";
const String kDataSourceDart = "_data_source$kDart";
const String kApiModel = 'api/model/';
const String kDataModel = '/data/model/';
const String kDomain = '/domain/';
const String kPresentationBloc = '/presentation/bloc/';
const String kCurlyBrackets = '{}';
const String kCurlyBracketsNewLine = '{\n}';
const String kAssetsImagesPath = 'assets/images/';
const String kImageTwoX = '/2.0x';
const String kImageThreeX = '/3.0x';
const String kMimePackage = 'package:mime/mime';

class PathHandler {
  String getDependencyPath() {
    String dependencyName = Utility.getPackageNameAndVersion();
    String pubCachePath;
    if (Platform.isWindows) {
      String userProfile = Platform.environment['USERPROFILE'] ?? '';
      pubCachePath = Platform.environment['PUB_CACHE'] ??
          path.join(userProfile, 'AppData', 'Local', 'Pub', 'Cache');
    } else {
      pubCachePath = Platform.environment['PUB_CACHE'] ??
          path.join(Platform.environment['HOME']!, '.pub-cache');
    }
    String dependencyPath =
        path.join(pubCachePath, 'hosted', 'pub.dev', dependencyName);

    return dependencyPath;
  }

  String getModelPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kModelPath";

  String getScreenPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kScreenPath";

  String getRepoImport({
    required String featureName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDomainRepositories${featureName}_repo.dart';";

  List<String> getMockDiImports({
    required String featureName,
  }) =>
      [
        "$kImportPackageWithName$kFeatures$featureName${kDataRepository}mock_${featureName}_repo_impl.dart';",
        "$kImportPackageWithName${kCore}di/injection_container.dart';",
      ];

  List<String> getInjectionContainerImports({
    required String featureName,
  }) =>
      [
        "$kImportPackageWithName$kFeatures$featureName$kDataDataSource${featureName}_data_source.dart';",
        "$kImportPackageWithName$kFeatures$featureName$kDataDataSource${featureName}_data_source_impl.dart';"
      ];

  String getProdDiGetterImport({
    required String featureName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDataRepository${featureName}_repo_impl.dart';";

  String getMockJsonPath({
    required String apiName,
  }) =>
      "$kAssetsMockJson$apiName$kJson";

  String getDataSourcePath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kDataDataSource$featureName$kDataSourceDart";

  String getApiResponseImport() =>
      "$kImportPackageWithName$kCore${kApiModel}api_response$kDart';";

  String getRequestModelImport({
    required String featureName,
    required String apiName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDataModel${apiName}_request_model$kDart';";

  String getMultipartFileRequestModelImport() =>
      "$kImportPackageWithName$kCore${kApiModel}file_upload$kRequestModel$kDart';";

  String getMimeTypeImport() => "$kImport '$kMimePackage$kDart';";

  String getDataSourceImplPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kDataDataSource${featureName}_data_source_impl$kDart";

  String getApiResponsePath() =>
      "$kImportPackageWithName$kCore${kApiModel}api_response$kDart';";

  String getApiConstantPath({
    required String featureName,
  }) =>
      "$kImportPackageWithName${kCore}const/api_constants$kDart';";

  String getRepoPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kDomainRepositories${featureName}_repo$kDart";

  String getFailureImport({
    required String featureName,
  }) =>
      "$kImportPackageWithName$kCore${kApiModel}failure$kDart';";

  String getSuccessImport({
    required String featureName,
  }) =>
      "$kImportPackageWithName$kCore${kApiModel}success$kDart';";

  String getDartzImport({
    required String featureName,
  }) =>
      "import 'package:dartz/dartz$kDart';";

  String getRepoImplPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kDataRepository${featureName}_repo_impl$kDart";

  String getRepoImplResponseModelImport({
    required String featureName,
    required String responseModelName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDataModel$responseModelName$kDart';";

  String getMockRepoImplPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName${kDataRepository}mock_${featureName}_repo_impl$kDart";

  String getMockApiConstantImport({
    required String featureName,
  }) =>
      "$kImportPackageWithName${kCore}const/mock_api_constants$kDart';";

  String getResponseModelImport({
    required String featureName,
    required String responseModelName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDataModel$responseModelName$kDart';";

  String getUseCasesPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName${kDomain}usecases";

  String getUseCaseRequestModelImport({
    required String featureName,
    required String useCaseName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDataModel${useCaseName}_request_model$kDart';";

  String getDIFilePath() => "${kLibCore}di/";

  String getUseCaseImport({
    required String featureName,
    required String useCaseName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName${kDomain}usecases/$useCaseName$kUseCasesPrefixUnderscore$kDart';";

  String getPagePath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName/${featureName}_page$kDart";

  String getInjectionContainerUseCaseImport({
    required String featureName,
    required String useCaseName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName${kDomain}usecases/$useCaseName$kUseCasesPrefixUnderscore$kDart';";

  String getInjectionContainerImport() =>
      "$kImportPackageWithName${kCore}di/injection_container.dart';";

  String getEventPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kPresentationBloc${featureName}_event$kDart";

  String getRequestModelImportFile({
    required String featureName,
    required String requestModelName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName$kDataModel$requestModelName$kDart';";

  String getStatePath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kPresentationBloc${featureName}_state$kDart";

  String getBlocPath({
    required String featureName,
  }) =>
      "$kLibFeatures$featureName$kPresentationBloc${featureName}_bloc$kDart";

  String getFlutterBlocPath({
    required String featureName,
  }) =>
      "import 'package:flutter_bloc/flutter_bloc$kDart';";

  String getUseCaseFileImport({
    required String featureName,
    required String useCaseFileName,
  }) =>
      "$kImportPackageWithName$kFeatures$featureName/domain/usecases/$useCaseFileName$kDart';";

  String getNoRequestImport() =>
      "$kImportPackageWithName${kCore}api/model/no_request$kDart';";

  String getApiBaseBlocEventImport({
    required String featureName,
  }) =>
      "$kImportPackageWithName${kCore}base/bloc/api_base_bloc/api_base_bloc_event$kDart';";

  String getCoreApiConstantPath() => "${kLibCore}const/api_constants.dart";
  String getCoreMockApiConstantPath() =>
      "${kLibCore}const/mock_api_constants.dart";
}
