import 'dart:io';

import 'package:jt_flutter_cli/core/code/code_handler.dart';
import 'package:jt_flutter_cli/core/const/command_const.dart';
import 'package:jt_flutter_cli/core/const/string_const.dart';
import 'package:jt_flutter_cli/core/path/path_handler.dart';
import 'package:jt_flutter_cli/core/util/model_util/helpers.dart';
import 'package:jt_flutter_cli/core/util/util.dart';
import 'package:jt_flutter_cli/feature/create_model/create_model_cli.dart';

class AddAPICallUtil {
  Future<String> generateRequestModel({
    required String featureName,
    required String apiName,
    required String apiMethodType,
  }) async {
    String requestModelName = kWhitespaces;

    if (apiMethodType == kGet) {
      Utility.stdoutWrite(text: kQueryParametersOrNot);
    } else {
      Utility.stdoutWrite(text: kRequestBodyOrNot);
    }
    String requestModelChoice = stdin.readLineSync()!.toLowerCase();

    if (requestModelChoice == kYes) {
      requestModelName = "$apiName$kRequestModel";
      Utility.stdoutWrite(text: kRequestJsonFilePath);
      String requestModelJsonFilePath = stdin.readLineSync()!;
      Utility.createFolder(path: '$kLib$kFeatures$featureName$kModelPath');
      CreateModel().generateDartModel(
        modelName: requestModelName,
        featureName: featureName,
        jsonFilePath: requestModelJsonFilePath,
      );
      if (apiMethodType == kMultipart) {
        String outputFileName = "$requestModelName$kDart";
        await findAndReplaceForMultipart(
            filePath: '$kLibFeatures$featureName$kDataModel$outputFileName');
        await Utility.addImport(
          filePath: '$kLibFeatures$featureName$kDataModel$outputFileName',
          importStatements: [kDartIoImport],
        );
      }
    }
    return requestModelName;
  }

  Future<String> generateResponseModel({
    required String featureName,
    required String apiName,
    required String apiMethodType,
  }) async {
    Utility.stdoutWrite(text: kResponseBodyOrNot);
    String responseModelChoice = stdin.readLineSync()!.toLowerCase();
    String responseModelName = kWhitespaces;
    if (responseModelChoice == kYes) {
      String modelName = "$apiName$kResponseModel";
      responseModelName = modelName;
      Utility.stdoutWrite(text: kResponseJsonFilePath);
      String responseModelJsonFilePath = stdin.readLineSync()!;

      ///Copy response json to assets/mock
      if (!Utility.doesFileExist(
        filePath: PathHandler().getMockJsonPath(
          apiName: apiName,
        ),
      )) {
        String mockJsonFileData =
            File(responseModelJsonFilePath).readAsStringSync();
        File mockJsonFile = await File(
          PathHandler().getMockJsonPath(
            apiName: apiName,
          ),
        ).create();
        mockJsonFile.writeAsStringSync(mockJsonFileData);
      }

      Utility.createFolder(
        path: PathHandler().getModelPath(
          featureName: featureName,
        ),
      );
      CreateModel().generateDartModel(
        modelName: modelName,
        featureName: featureName,
        jsonFilePath: responseModelJsonFilePath,
      );
    }
    return responseModelName;
  }

  Future<void> createDataSourceAndDataSourceImpl({
    required String featureName,
    required String requestModelName,
    required String apiName,
    required String apiMethodName,
  }) async {
    String pascalCaseApiName = Utility.toPascalCase(
      input: apiName,
    );
    String dataSourceDestinationPath = PathHandler().getDataSourcePath(
      featureName: featureName,
    );
    if (Utility.doesFileExist(filePath: dataSourceDestinationPath)) {
      final importDataSourceStatements = [
        PathHandler().getApiResponseImport(),
      ];

      List<String> dataSourceAppendLinesDataSource = [];
      if (requestModelName.isNotEmpty) {
        importDataSourceStatements.add(
          PathHandler().getRequestModelImport(
            featureName: featureName,
            apiName: apiName,
          ),
        );
        dataSourceAppendLinesDataSource.add(
          CodeHandler().getDataSourceStatementWithRequestModel(
            pascalCaseApiName: pascalCaseApiName,
            requestModelName: requestModelName,
          ),
        );
      } else {
        dataSourceAppendLinesDataSource = [
          CodeHandler().getDataSourceStatementWithoutRequestModel(
            pascalCaseApiName: pascalCaseApiName,
          )
        ];
      }
      await fileUpdate(
        destinationPath: dataSourceDestinationPath,
        apiResponseImport: importDataSourceStatements,
        dataSourceAppendLines: dataSourceAppendLinesDataSource,
      );
    }

    /// update data_source_impl.dart to add imports statement and function method
    String destinationPathDataSourceImpl =
        PathHandler().getDataSourceImplPath(featureName: featureName);
    if (Utility.doesFileExist(filePath: destinationPathDataSourceImpl)) {
      final importDataSourceImplStatements = [
        PathHandler().getApiResponsePath(),
        PathHandler().getApiConstantPath(featureName: featureName)
      ];
      List<String> dataSourceAppendLinesDataSourceImpl = [];

      if (requestModelName.isNotEmpty) {
        importDataSourceImplStatements.add(
          PathHandler().getRequestModelImport(
            featureName: featureName,
            apiName: apiName,
          ),
        );
        if (apiMethodName == kGet) {
          dataSourceAppendLinesDataSourceImpl = CodeHandler()
              .getDataSourceImplStatementForGetMethodWithRequestModel(
            pascalCaseApiName: pascalCaseApiName,
            requestModelName: requestModelName,
            apiMethodName: apiMethodName,
            apiName: apiName,
          );
        } else if (apiMethodName == kPost) {
          dataSourceAppendLinesDataSourceImpl = CodeHandler()
              .getDataSourceImplStatementForPostMethodWithRequestModel(
            pascalCaseApiName: pascalCaseApiName,
            requestModelName: requestModelName,
            apiMethodName: apiMethodName,
            apiName: apiName,
          );
        } else if (apiMethodName == kMultipart) {
          String requestModelPath =
              '${PathHandler().getModelPath(featureName: featureName)}/$apiName$kRequestModel$kDart';
          List<String> multipartFileKeys = await findMultipartFileKeys(
            filePath: requestModelPath,
          );
          importDataSourceImplStatements
              .add(PathHandler().getMultipartFileRequestModelImport());
          importDataSourceImplStatements.add(PathHandler().getMimeTypeImport());
          dataSourceAppendLinesDataSourceImpl =
              CodeHandler().getDataSourceImplStatementForMultipartMethod(
            pascalCaseApiName: pascalCaseApiName,
            requestModelName: requestModelName,
            apiMethodName: apiMethodName,
            apiName: apiName,
            multipartFileKeys: multipartFileKeys,
          );
        }
      } else {
        if (apiMethodName == kGet) {
          dataSourceAppendLinesDataSourceImpl = CodeHandler()
              .getDataSourceImplStatementForGetMethodWithoutRequestModel(
            pascalCaseApiName: pascalCaseApiName,
            apiMethodName: apiMethodName,
            apiName: apiName,
          );
        } else if (apiMethodName == kPost) {
          dataSourceAppendLinesDataSourceImpl = CodeHandler()
              .getDataSourceImplStatementForPostMethodWithoutRequestModel(
            pascalCaseApiName: pascalCaseApiName,
            apiMethodName: apiMethodName,
            apiName: apiName,
          );
        }
      }
      await fileUpdate(
        destinationPath: destinationPathDataSourceImpl,
        apiResponseImport: importDataSourceImplStatements,
        dataSourceAppendLines: dataSourceAppendLinesDataSourceImpl,
      );
    }
  }

  Future<void> createRepoAndRepoImpl({
    required String featureName,
    required String requestModelName,
    required String responseModelName,
    required String apiName,
    required String apiMethodName,
  }) async {
    String pascalCaseApiName = Utility.toPascalCase(input: apiName);

    /// update repo.dart to add imports statement and function signature
    String destinationPathRepo =
        PathHandler().getRepoPath(featureName: featureName);
    if (Utility.doesFileExist(filePath: destinationPathRepo)) {
      final importRepoStatements = [
        PathHandler().getFailureImport(featureName: featureName),
        PathHandler().getSuccessImport(featureName: featureName),
        PathHandler().getDartzImport(featureName: featureName)
      ];
      List<String> repoAppendLines = [];

      if (requestModelName.isNotEmpty) {
        importRepoStatements.add(
          PathHandler().getRequestModelImport(
            featureName: featureName,
            apiName: apiName,
          ),
        );
        repoAppendLines = [
          CodeHandler().getRepoStatementWithRequestModel(
            requestModelName: requestModelName,
            pascalCaseApiName: pascalCaseApiName,
          ),
        ];
      } else {
        repoAppendLines = [
          CodeHandler().getRepoStatementWithoutRequestModel(
            pascalCaseApiName: pascalCaseApiName,
          ),
        ];
      }
      await fileUpdate(
        destinationPath: destinationPathRepo,
        apiResponseImport: importRepoStatements,
        dataSourceAppendLines: repoAppendLines,
      );
    }

    /// update repoImpl.dart to add imports statement and function signature
    String destinationPathRepoImpl =
        PathHandler().getRepoImplPath(featureName: featureName);
    if (Utility.doesFileExist(filePath: destinationPathRepoImpl)) {
      List<String> importRepoImplStatements = [
        PathHandler().getDartzImport(
          featureName: featureName,
        ),
        PathHandler().getApiResponsePath(),
        PathHandler().getFailureImport(
          featureName: featureName,
        ),
        PathHandler().getSuccessImport(
          featureName: featureName,
        ),
      ];

      if (responseModelName.isNotEmpty) {
        importRepoImplStatements.add(
          PathHandler().getRepoImplResponseModelImport(
            featureName: featureName,
            responseModelName: responseModelName,
          ),
        );
      }

      List<String> repoImplAppendLines = [];

      if (requestModelName.isNotEmpty) {
        importRepoImplStatements.add(
          PathHandler().getRequestModelImport(
            featureName: featureName,
            apiName: apiName,
          ),
        );
        repoImplAppendLines =
            CodeHandler().getRepoImplStatementWithRequestModel(
          pascalCaseApiName: pascalCaseApiName,
          requestModelName: requestModelName,
        );
      } else {
        repoImplAppendLines =
            CodeHandler().getRepoImplStatementWithoutRequestModel(
          pascalCaseApiName: pascalCaseApiName,
        );
      }
      if (responseModelName.isNotEmpty) {
        repoImplAppendLines.add(
          CodeHandler().getRepoStatementWithResponseModel(
            responseModelName: responseModelName,
          ),
        );
      } else {
        repoImplAppendLines.add(
          CodeHandler().getRepoStatementWithoutResponseModel(),
        );
      }

      await fileUpdate(
        destinationPath: destinationPathRepoImpl,
        apiResponseImport: importRepoImplStatements,
        dataSourceAppendLines: repoImplAppendLines,
      );
    }

    /// update mockRepoImpl.dart to add imports statement and function signature
    String destinationPathMockRepoImpl =
        PathHandler().getMockRepoImplPath(featureName: featureName);
    if (Utility.doesFileExist(filePath: destinationPathMockRepoImpl)) {
      List<String> importMockRepoImplStatements = [
        PathHandler().getDartzImport(featureName: featureName),
        PathHandler().getApiResponsePath(),
        PathHandler().getFailureImport(featureName: featureName),
        PathHandler().getSuccessImport(featureName: featureName),
        PathHandler().getMockApiConstantImport(featureName: featureName),
      ];

      if (responseModelName.isNotEmpty) {
        importMockRepoImplStatements.add(
          PathHandler().getResponseModelImport(
            featureName: featureName,
            responseModelName: responseModelName,
          ),
        );
      }

      List<String> mockRepoImplAppendLines = [];

      if (requestModelName.isNotEmpty) {
        importMockRepoImplStatements.add(
          PathHandler().getRequestModelImport(
            featureName: featureName,
            apiName: apiName,
          ),
        );
        mockRepoImplAppendLines =
            CodeHandler().getMockRepoImplStatementWithRequestModel(
          pascalCaseApiName: pascalCaseApiName,
          requestModelName: requestModelName,
          apiName: apiName,
        );
      } else {
        mockRepoImplAppendLines =
            CodeHandler().getMockRepoImplStatementWithoutRequestModel(
          pascalCaseApiName: pascalCaseApiName,
          apiName: apiName,
        );
      }
      if (responseModelName.isNotEmpty) {
        mockRepoImplAppendLines.add(
          CodeHandler().getMockRepoStatementWithResponseModel(
            responseModelName: responseModelName,
          ),
        );
      } else {
        mockRepoImplAppendLines.add(
          CodeHandler().getMockRepoStatementWithoutResponseModel(),
        );
      }

      await fileUpdate(
        destinationPath: destinationPathMockRepoImpl,
        apiResponseImport: importMockRepoImplStatements,
        dataSourceAppendLines: mockRepoImplAppendLines,
      );
    }
  }

  Future<void> createUsaCase({
    required String featureName,
    required String useCaseName,
    required String requestModelName,
    required String apiMethod,
  }) async {
    String destinationPath =
        PathHandler().getUseCasesPath(featureName: featureName);
    String destinationFilePath = '$destinationPath/$useCaseName$kDart';
    if (Utility.doesFileExist(filePath: destinationFilePath)) {
      Utility.errorPrinter(
          text: '$kFileAT $destinationFilePath $kExistChooseDifferentUseCase');
      exit(1);
    }

    String sourceFolderPath =
        Utility.getDirFromCliPackage(sourceFolderPath: kUseCaseBasicStructure);

    try {
      await Utility.copyFolder(
        sourcePath: sourceFolderPath,
        destinationPath: destinationPath,
      );
      await Utility.renameFilesWithSubstring(
        path: destinationPath,
        substring: kBasicStructureLowerCase,
        newName: useCaseName,
      );
      String targetFileName = '$useCaseName$kUseCasesPrefixUnderscore$kDart';
      await Utility.processFiles(
        path: destinationPath,
        substring: kStartComment,
        replacement: kWhitespaces,
        isPascalCase: false,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kEndComment,
        replacement: kWhitespaces,
        isPascalCase: false,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kBasicStructurePascalCase,
        replacement: useCaseName,
        isPascalCase: true,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kBasicStructureCamelCase,
        replacement: useCaseName,
        isPascalCase: true,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kCommonCap,
        replacement: camelCase(featureName),
        isPascalCase: false,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
          path: destinationPath,
          substring: kBasicStructureLowerCase,
          replacement: useCaseName,
          isPascalCase: false,
          targetFileName: targetFileName);
      await Utility.processFiles(
        path: destinationPath,
        substring: kCommon.toLowerCase(),
        replacement: Utility.camelCaseFirstLower(input: featureName),
        isPascalCase: false,
        targetFileName: targetFileName,
      );
      String? name = Utility.getPubspecInfo();
      if (name != null) {
        await Utility.processFiles(
          path: destinationPath,
          substring: kFlutterCleanArchitecture,
          replacement: name,
          isPascalCase: false,
          targetFileName: targetFileName,
        );
      }
      String useCaseFilePath =
          '$destinationPath/$useCaseName$kUseCasesPrefixUnderscore$kDart';
      if (requestModelName.isNotEmpty) {
        await Utility.processFiles(
          path: destinationPath,
          substring: kParams,
          replacement: Utility.camelCaseFirstLower(
            input: requestModelName,
          ),
          isPascalCase: false,
          targetFileName: targetFileName,
        );
        await Utility.processFiles(
          path: destinationPath,
          substring: kNoRequest,
          replacement: requestModelName,
          isPascalCase: true,
          targetFileName: targetFileName,
        );
        await Utility.processFiles(
          path: destinationPath,
          substring: '$kCall${camelCase(useCaseName)}()',
          replacement:
              "$kCall${camelCase(useCaseName)}(${Utility.camelCaseFirstLower(input: requestModelName)})",
          isPascalCase: false,
          targetFileName: targetFileName,
        );
        final usecaseFileImport = [
          PathHandler().getUseCaseRequestModelImport(
            featureName: featureName,
            useCaseName: useCaseName,
          )
        ];
        await Utility.addImport(
          filePath: useCaseFilePath,
          importStatements: usecaseFileImport,
        );
      }

      String pascalCase = Utility.toPascalCase(
        input: useCaseName,
      );
      final filePath = PathHandler().getDIFilePath();
      final usecaseImport = PathHandler().getUseCaseImport(
        featureName: featureName,
        useCaseName: useCaseName,
      );
      final injectionContainerImports = [
        usecaseImport,
      ];
      final injectionContainerStatements =
          CodeHandler().getInjectionContainerStatements(
        pascalCase: pascalCase,
        featureName: featureName,
      );
      await Utility().addImportAndChangesInFile(
        filePath: '$filePath$kInjectionContainer',
        importStatements: injectionContainerImports,
        methodCalls: injectionContainerStatements,
      );
      final repoFileImport = [
        PathHandler().getRepoImport(
          featureName: featureName,
        )
      ];
      await Utility.addImport(
        filePath: useCaseFilePath,
        importStatements: repoFileImport,
      );

      ///Add use_case in page bloc
      await Utility.addImport(
        filePath: PathHandler().getPagePath(
          featureName: featureName,
        ),
        importStatements: [
          PathHandler().getInjectionContainerImport(),
        ],
      );
      await Utility.findAndReplace(
        filePath: PathHandler().getPagePath(
          featureName: featureName,
        ),
        searchString: CodeHandler().getPageSearchString(),
        replacement: CodeHandler().getPageReplacementString(
          useCaseName: Utility.camelCaseFirstLower(input: useCaseName),
        ),
      );
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
  }

  Future<void> updateBlocEventAndStates({
    required String featureName,
    required String requestModelName,
    required String responseModelName,
    required String apiName,
    required String apiMethodName,
    required String shimmerChoice,
  }) async {
    String featureNameString = featureName.toLowerCase();
    String pascalCaseApiName = Utility.toPascalCase(
      input: apiName,
    );
    String featureNamePascalCase =
        Utility.toPascalCase(input: featureNameString);
    String destinationPathEvent = PathHandler().getEventPath(
      featureName: featureName,
    );
    List<String> eventImportStatements = [];
    if (requestModelName.isNotEmpty) {
      eventImportStatements.add(
        PathHandler().getRequestModelImportFile(
          featureName: featureName,
          requestModelName: requestModelName,
        ),
      );
    }

    if (Utility.doesFileExist(filePath: destinationPathEvent)) {
      List<String> eventAppendLines = [];
      if (requestModelName.isNotEmpty) {
        eventAppendLines.add(
          CodeHandler().getEventStatementWithResponseModel(
            pascalCaseApiName: pascalCaseApiName,
            featureNamePascalCase: featureNamePascalCase,
            requestModelName: requestModelName,
          ),
        );
      } else {
        eventAppendLines.add(
          CodeHandler().getEventStatementWithoutResponseModel(
            pascalCaseApiName: pascalCaseApiName,
            featureNamePascalCase: featureNamePascalCase,
          ),
        );
      }

      await fileUpdate(
        destinationPath: destinationPathEvent,
        apiResponseImport: eventImportStatements,
        dataSourceAppendLines: eventAppendLines,
        isAppendEOF: true,
      );
    }

    /// update state.dart to add imports statement and function method
    String destinationPathState =
        PathHandler().getStatePath(featureName: featureName);
    if (Utility.doesFileExist(filePath: destinationPathState)) {
      List<String> importStateStatements = [];
      if (responseModelName.isNotEmpty) {
        importStateStatements = [
          PathHandler().getResponseModelImport(
            featureName: featureName,
            responseModelName: responseModelName,
          )
        ];
      }

      List<String> stateSourceAppendLines = [];

      if (responseModelName.isNotEmpty) {
        stateSourceAppendLines =
            CodeHandler().getStateStatementsWithRequestModel(
          pascalCaseApiName: pascalCaseApiName,
          responseModelName: responseModelName,
          featureNamePascalCase: featureNamePascalCase,
        );
      } else {
        stateSourceAppendLines =
            CodeHandler().getStateStatementsWithoutRequestModel(
          pascalCaseApiName: pascalCaseApiName,
          featureNamePascalCase: featureNamePascalCase,
        );
      }

      if (shimmerChoice == kYes.toLowerCase()) {
        stateSourceAppendLines.add(
          CodeHandler().getStateStatementShimmerChoiceY(
            pascalCaseApiName: pascalCaseApiName,
            featureNamePascalCase: featureNamePascalCase,
          ),
        );
      }
      await fileUpdate(
        destinationPath: destinationPathState,
        apiResponseImport: importStateStatements,
        dataSourceAppendLines: stateSourceAppendLines,
        isAppendEOF: true,
      );
    }

    /// update bloc.dart to add imports statement and function signature
    String useCaseName = "$apiName$kUseCasePascal";
    String useCaseFileName = "$apiName$kUseCasesPrefixUnderscore";
    String destinationPathBloc =
        PathHandler().getBlocPath(featureName: featureName);
    if (Utility.doesFileExist(filePath: destinationPathBloc)) {
      List<String> importBlocImplStatements = [
        "",
        PathHandler().getDartzImport(featureName: featureName),
        PathHandler().getFailureImport(featureName: featureName),
        PathHandler().getSuccessImport(featureName: featureName),
        PathHandler().getFlutterBlocPath(featureName: featureName),
        PathHandler().getUseCaseFileImport(
          featureName: featureName,
          useCaseFileName: useCaseFileName,
        ),
        requestModelName.isEmpty ? PathHandler().getNoRequestImport() : ""
      ];

      if (responseModelName.isNotEmpty) {
        importBlocImplStatements.add(
          PathHandler().getResponseModelImport(
            featureName: featureName,
            responseModelName: responseModelName,
          ),
        );
      }

      List<String> blockAppendLines = [];

      String usCaseEvent = CodeHandler().getUseCaseEventNoRequest(
        useCaseName: useCaseName,
      );
      if (requestModelName.isNotEmpty) {
        usCaseEvent = CodeHandler().getUseCaseEventWithRequestModel(
          useCaseName: useCaseName,
        );
      }

      if (shimmerChoice == kYes.toLowerCase()) {
        blockAppendLines = CodeHandler().getBlocStatementsShimmerChoiceY(
          pascalCaseApiName: pascalCaseApiName,
          apiName: apiName,
          usCaseEvent: usCaseEvent,
        );
      } else {
        importBlocImplStatements.add(
          PathHandler().getApiBaseBlocEventImport(
            featureName: featureName,
          ),
        );
        blockAppendLines = CodeHandler().getBlocStatementsShimmerChoiceN(
          pascalCaseApiName: pascalCaseApiName,
          apiName: apiName,
          usCaseEvent: usCaseEvent,
        );
      }
      if (responseModelName.isNotEmpty) {
        String responseModel = camelCase(responseModelName);
        String responseName =
            responseModel[0].toLowerCase() + responseModel.substring(1);
        blockAppendLines.add(
          CodeHandler().getBlocStatementWithResponseModel(
            responseModelName: responseModelName,
            responseName: responseName,
            pascalCaseApiName: pascalCaseApiName,
          ),
        );
      } else {
        blockAppendLines.add(
          CodeHandler().getBlocStatementWithoutResponseModel(
            pascalCaseApiName: pascalCaseApiName,
          ),
        );
      }

      await fileUpdate(
        destinationPath: destinationPathBloc,
        apiResponseImport: importBlocImplStatements,
        dataSourceAppendLines: blockAppendLines,
      );

      List<String> linesToAddAfterSuperApiBaseBloc =
          CodeHandler().getAddBlocAfterSuperApiBaseBloc(
        pascalCaseApiName: pascalCaseApiName,
        apiName: apiName,
      );

      await appendLine(
        destinationPathBloc,
        linesToAddAfterSuperApiBaseBloc,
        CodeHandler().getBlocAppendAfterContainString(
          pascalCaseApiName: pascalCaseApiName,
        ),
      );

      List<String> linesToAddBeforeSuperApiBaseBloc =
          CodeHandler().getAddBlocBeforeSuperApiBaseBloc(
        useCaseName: useCaseName,
      );

      await appendLine(
        destinationPathBloc,
        linesToAddBeforeSuperApiBaseBloc,
        CodeHandler().getBlocSuperApiBaseBloc(
          pascalCaseApiName: pascalCaseApiName,
        ),
        isAfter: false,
      );

      List<String> linesToAddUseCaseDeclaration = [
        "    $kFinal ${camelCase(useCaseName)} ${Utility.camelCaseFirstLower(input: useCaseName)};",
      ];

      await appendLine(
        destinationPathBloc,
        linesToAddUseCaseDeclaration,
        CodeHandler().getExtendsBaseBloc(
          pascalCaseApiName: pascalCaseApiName,
        ),
      );
    }
  }

  Future<void> fileUpdate({
    required String destinationPath,
    required List<String> apiResponseImport,
    required List<String> dataSourceAppendLines,
    bool isAppendEOF = false,
  }) async {
    if (Utility.doesFileExist(filePath: destinationPath)) {
      try {
        if (apiResponseImport.isNotEmpty) {
          await Utility.addImport(
            filePath: destinationPath,
            importStatements: apiResponseImport,
          );
        }
        if (isAppendEOF) {
          await Utility.appendLinesToFile(
            filePath: destinationPath,
            linesToAppend: dataSourceAppendLines,
          );
        } else {
          await Utility.findAndReplace(
            filePath: destinationPath,
            searchString: kCurlyBrackets,
            replacement: kCurlyBracketsNewLine,
          );
          await Utility.addCodeAboveBraces(
            filePath: destinationPath,
            linesToAdd: dataSourceAppendLines,
          );
        }
      } catch (e) {
        Utility.errorPrinter(text: '$kError $e');
      }
    } else {
      Utility.errorPrinter(
          text: '$kError $kFileAT $destinationPath $kNotExist');
    }
  }

  Future<void> appendLine(
    String filePath,
    List<String> linesToAdd,
    String appendAfterContainString, {
    bool isAfter = true,
  }) async {
    final file = File(filePath);
    final lines = await file.readAsLines();
    bool found = false;

    for (int i = 0; i < lines.length; i++) {
      if (lines[i].contains(appendAfterContainString) && !found) {
        if (isAfter) {
          lines.insertAll(i + 1, linesToAdd);
        } else {
          lines.insertAll(i - 1, linesToAdd);
        }
        found = true;
        break;
      }
    }
    if (!found) {
      return;
    }
    await file.writeAsString(lines.join('\n'));
  }

  Future<void> apiConstantFileUpdate({
    required String apiType,
    required String apiName,
    required String endPoint,
  }) async {
    final apiConstantFilePath = PathHandler().getCoreApiConstantPath();
    final apiConstantLineAppend = [
      CodeHandler().getConstantStringAppend(
        apiType: apiType,
        apiName: apiName,
        endPoint: endPoint,
      )
    ];
    try {
      await Utility.appendLinesToFile(
        filePath: apiConstantFilePath,
        linesToAppend: apiConstantLineAppend,
      );
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
  }

  Future<void> mockApiConstantFileUpdate({
    required String apiName,
  }) async {
    String endPoint = "$apiName$kJson";
    final apiConstantFilePath = PathHandler().getCoreMockApiConstantPath();
    final apiConstantLineAppend = [
      CodeHandler().getMockConstantStringAppend(
        apiName: apiName,
        endPoint: endPoint,
      )
    ];
    try {
      await Utility.appendLinesToFile(
        filePath: apiConstantFilePath,
        linesToAppend: apiConstantLineAppend,
      );
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
  }

  static Future<void> findAndReplaceForMultipart({
    required String filePath,
  }) async {
    try {
      final file = File(filePath);
      List<String> lines = await file.readAsLines();
      for (int i = 0; i < lines.length; i++) {
        if (lines[i].contains("String?") && lines[i].contains("File")) {
          lines[i] = lines[i].replaceAll("String?", "File?");
        }
        if (lines[i].contains("_file")) {
          lines[i] = lines[i].replaceAll("_file", "");
        }
      }
      await file.writeAsString(lines.join('\n'));
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
  }

  Future<List<String>> findMultipartFileKeys({
    required String filePath,
  }) async {
    List<String> multipartKeys = [];
    try {
      final file = File(filePath);
      List<String> lines = await file.readAsLines();
      for (int i = 0; i < lines.length; i++) {
        if (lines[i].contains("json") && lines[i].contains("File")) {
          String parameter = lines[i].split("=").last;
          // print("Index: $i\n");
          // print(
          //     "File?: ${Utility().convertCamelcaseStringToUnderscore(input: parameter.replaceFirst("File?", ""))}\n");
          // print(
          //     "Empty: ${Utility().convertCamelcaseStringToUnderscore(input: parameter.replaceFirst("File?", "")).replaceAll(" ", "")}\n");
          // print(
          //     ";: ${Utility().convertCamelcaseStringToUnderscore(input: parameter.replaceFirst("File?", "")).replaceAll(" ", "").replaceAll(";", "")}\n");
          // print(
          //     "File: ${Utility().convertCamelcaseStringToUnderscore(input: parameter.replaceFirst("File?", "")).replaceAll(" ", "").replaceAll(";", "").replaceFirst("File", "")}\n");

          String multipartKey = parameter
              .replaceAll("json", "")
              .replaceAll("[", "")
              .replaceAll("]", "")
              .replaceAll(";", "")
              .replaceAll(" ", "")
              .replaceAll("'", "");
          multipartKeys.add(multipartKey);
        }
      }
      await file.writeAsString(lines.join('\n'));
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
    return multipartKeys;
  }
}
