import 'dart:io';

import 'package:jt_flutter_cli/core/const/command_const.dart';
import 'package:jt_flutter_cli/core/const/string_const.dart';
import 'package:jt_flutter_cli/core/util/util.dart';
import 'package:jt_flutter_cli/feature/add_api_call/add_api_call_util.dart';

class AddAPI {
  Future<void> generateApiCallCode({
    required String apiMethod,
    required String apiEndPoint,
    required String apiName,
  }) async {
    try {
      late String method;
      if (apiMethod == kGetApiCall) {
        method = kGet;
      }
      if (apiMethod == kPostApiCall) {
        method = kPost;
      }
      if (apiMethod == kMultipartApiCall) {
        method = kMultipart;
      }
      Utility.stdoutWrite(text: kEnterFeatureName);
      String featureChoice = stdin.readLineSync()!.toLowerCase();
      String requestModelName = await AddAPICallUtil().generateRequestModel(
        featureName: featureChoice,
        apiName: apiName,
        apiMethodType: method,
      );

      String responseModelName = await AddAPICallUtil().generateResponseModel(
        featureName: featureChoice,
        apiName: apiName,
        apiMethodType: method,
      );

      Utility.stdoutWrite(text: kAddShimmerOrNot);
      String shimmerChoice = stdin.readLineSync()!.toLowerCase();

      await addAPICall(
        apiType: method.toLowerCase(),
        endPoint: apiEndPoint,
        apiName: apiName,
        featureName: featureChoice,
        responseModelName: responseModelName,
        shimmerChoice: shimmerChoice,
        requestModelName: requestModelName,
        apiMethodName: method,
      );
      Utility.successPrinter(
        text: '$kAddedApiTo $featureChoice $kFeatureString',
      );
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
      exit(1);
    }
  }

  Future<void> addAPICall({
    required String apiType,
    required String endPoint,
    required String apiName,
    required String featureName,
    required String responseModelName,
    required String shimmerChoice,
    required String requestModelName,
    required String apiMethodName,
  }) async {
    /// update api_constants.dart file to add end point
    await AddAPICallUtil().apiConstantFileUpdate(
      apiType: apiType,
      apiName: apiName,
      endPoint: endPoint,
    );

    /// update api_constants.dart file to add end point
    await AddAPICallUtil().mockApiConstantFileUpdate(
      apiName: apiName,
    );

    /// update data_source.dart and data_source_impl to add imports statement and function signature
    await AddAPICallUtil().createDataSourceAndDataSourceImpl(
      featureName: featureName,
      requestModelName: requestModelName,
      apiName: apiName,
      apiMethodName: apiMethodName,
    );

    /// update repo.dart and repo_impl.dart to add imports statement and function signature
    await AddAPICallUtil().createRepoAndRepoImpl(
      featureName: featureName,
      requestModelName: requestModelName,
      responseModelName: responseModelName,
      apiName: apiName,
      apiMethodName: apiMethodName,
    );

    /// update a usecase
    await AddAPICallUtil().createUsaCase(
      featureName: featureName,
      useCaseName: apiName,
      requestModelName: requestModelName,
      apiMethod: apiMethodName,
    );

    /// update bloc ,event and state to add imports statement and function signature
    await AddAPICallUtil().updateBlocEventAndStates(
      featureName: featureName,
      requestModelName: requestModelName,
      responseModelName: responseModelName,
      apiName: apiName,
      apiMethodName: apiMethodName,
      shimmerChoice: shimmerChoice,
    );
  }
}
