import 'dart:async';
import 'dart:io';

import 'package:jt_flutter_cli/core/code/code_handler.dart';
import 'package:jt_flutter_cli/core/const/string_const.dart';
import 'package:jt_flutter_cli/core/path/path_handler.dart';
import 'package:jt_flutter_cli/core/util/util.dart';

import '../../core/const/command_const.dart';

class CreateWidget {
  Future<void> createWidget({
    required String widgetName,
  }) async {
    Utility.stdoutWrite(text: kCommonOrFeature);
    String choice = stdin.readLineSync()!.toLowerCase();

    if (choice == kCommon) {
      await createWidgetProcess(
        widgetName: widgetName,
        destinationPath: kCommonWidgetPath,
        isCommon: true,
      );
    } else if (choice == kFeatureCommand) {
      Utility.stdoutWrite(text: kEnterYourFeatureName);
      String featureName = stdin.readLineSync()!.toLowerCase();
      Directory destinationDirectory = Directory('$kLibFeatures$featureName');
      if (destinationDirectory.existsSync()) {
        await createWidgetProcess(
          widgetName: widgetName,
          destinationPath: PathHandler().getScreenPath(
            featureName: featureName,
          ),
          isCommon: false,
        );
      } else {
        Utility.errorPrinter(
          text: '$kFeatureLabel $featureName $kDoesNotExist',
        );
      }
    } else {
      Utility.errorPrinter(text: kCommonOrFeatureError);
      exit(1);
    }
  }

  Future<void> createWidgetProcess({
    required String widgetName,
    required String destinationPath,
    required bool isCommon,
  }) async {
    String destinationFilePath = destinationPath;
    if (!isCommon) {
      destinationFilePath = "$destinationFilePath$kWidgetPath";
    }
    destinationFilePath = "$destinationFilePath$widgetName$kDart";

    if (Utility.doesFileExist(filePath: destinationFilePath)) {
      Utility.errorPrinter(
        text: '$widgetName $kWidgetAlreadyExistError',
      );
      exit(1);
    }

    try {
      destinationPath = isCommon
          ? destinationPath
          : '$destinationPath$kWidgetWithoutEndSlashPath';
      await Utility.copyFolder(
        sourcePath: Utility.getDirFromCliPackage(
          sourceFolderPath: kWidgetSnippetPath,
        ),
        destinationPath: destinationPath,
      );
      await Utility.renameFilesWithSubstring(
        path: destinationPath,
        substring: kBasicWidgetLowerCase,
        newName: widgetName,
      );
      String targetFileName = isCommon ? '$widgetName$kDart' : kWhitespaces;
      await Utility.processFiles(
        path: destinationPath,
        substring: kStartComment,
        replacement: kWhitespaces,
        isPascalCase: false,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kEndComment,
        replacement: kWhitespaces,
        isPascalCase: false,
        targetFileName: targetFileName,
      );
      await Utility.processFiles(
        path: destinationPath,
        substring: kBasicWidget,
        replacement: widgetName,
        isPascalCase: true,
        targetFileName: targetFileName,
      );
      String? name = Utility.getPubspecInfo();
      if (name != null) {
        await Utility.processFiles(
          path: destinationPath,
          substring: kFlutterCleanArchitecture,
          replacement: name,
          isPascalCase: false,
          targetFileName: targetFileName,
        );
      }
      Utility.successPrinter(
        text: '$kWidgetLabel $widgetName $kCreatedAt $destinationPath',
      );
    } catch (e) {
      Utility.errorPrinter(text: '$kError $e');
    }
  }
}
